import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Catalog, Section
from ..serializers.v1 import SectionSerializer
from ..validators import SectionUniqueURIValidator


def test_unique_uri_validator_create(db):
    SectionUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'catalog': Catalog.objects.first()
    })


def test_unique_uri_validator_create_error(db):
    catalog = Catalog.objects.first()

    with pytest.raises(ValidationError):
        SectionUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': catalog.sections.first().key,
            'catalog': catalog
        })


def test_unique_uri_validator_update(db):
    section = Section.objects.first()

    SectionUniqueURIValidator(section)({
        'uri_prefix': section.uri_prefix,
        'key': section.key,
        'catalog': section.catalog
    })


def test_unique_uri_validator_update_error(db):
    section = Section.objects.first()

    with pytest.raises(ValidationError):
        SectionUniqueURIValidator(section)({
            'uri_prefix': section.uri_prefix,
            'key': section.catalog.sections.last().key,
            'catalog': section.catalog
        })


def test_unique_uri_validator_serializer_create(db):
    validator = SectionUniqueURIValidator()
    validator.set_context(SectionSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'catalog': Catalog.objects.first()
    })


def test_unique_uri_validator_serializer_create_error(db):
    catalog = Catalog.objects.first()

    validator = SectionUniqueURIValidator()
    validator.set_context(SectionSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': catalog.sections.last().key,
            'catalog': catalog
        })


def test_unique_uri_validator_serializer_update(db):
    section = Section.objects.first()

    validator = SectionUniqueURIValidator()
    validator.set_context(SectionSerializer(instance=section))

    validator({
        'uri_prefix': section.uri_prefix,
        'key': section.key,
        'catalog': section.catalog
    })


def test_unique_uri_validator_serializer_update_error(db):
    section = Section.objects.first()

    validator = SectionUniqueURIValidator()
    validator.set_context(SectionSerializer(instance=section))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': section.uri_prefix,
            'key': section.catalog.sections.last().key,
            'catalog': section.catalog
        })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import rdmo.core.models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0003_condition'),
    ]

    operations = [
        migrations.CreateModel(
            name='VerboseName',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name_en', models.CharField(max_length=256)),
                ('name_de', models.CharField(max_length=256)),
                ('name_plural_en', models.CharField(max_length=256)),
                ('name_plural_de', models.CharField(max_length=256)),
                ('attribute_entity', models.OneToOneField(to='domain.AttributeEntity', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'VerboseName',
                'verbose_name_plural': 'VerboseNames',
            },
            bases=(models.Model, rdmo.core.models.TranslationMixin),
        ),
    ]



import logging

from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from django.views.generic import UpdateView
from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin
from rdmo.questions.models import Catalog
from rdmo.tasks.models import Task
from rdmo.views.models import View

from ..forms import (ProjectForm, ProjectUpdateCatalogForm,
                     ProjectUpdateInformationForm, ProjectUpdateParentForm,
                     ProjectUpdateTasksForm, ProjectUpdateViewsForm)
from ..mixins import ProjectImportMixin
from ..models import Project

logger = logging.getLogger(__name__)


class ProjectUpdateView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectForm
    permission_required = 'projects.change_project_object'

    def get_form_kwargs(self):
        catalogs = Catalog.objects.filter_current_site() \
                                  .filter_group(self.request.user) \
                                  .filter_availability(self.request.user)
        projects = Project.objects.filter_user(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'catalogs': catalogs,
            'projects': projects
        })
        return form_kwargs


class ProjectUpdateInformationView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectUpdateInformationForm
    permission_required = 'projects.change_project_object'


class ProjectUpdateCatalogView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectUpdateCatalogForm
    permission_required = 'projects.change_project_object'

    def get_form_kwargs(self):
        catalogs = Catalog.objects.filter_current_site() \
                                  .filter_group(self.request.user) \
                                  .filter_availability(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'catalogs': catalogs
        })
        return form_kwargs


class ProjectUpdateTasksView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectUpdateTasksForm
    permission_required = 'projects.change_project_object'

    def get_form_kwargs(self):
        tasks = Task.objects.filter_current_site() \
                            .filter_catalog(self.object.catalog) \
                            .filter_group(self.request.user) \
                            .filter_availability(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'tasks': tasks
        })
        return form_kwargs


class ProjectUpdateViewsView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectUpdateViewsForm
    permission_required = 'projects.change_project_object'

    def get_form_kwargs(self):
        views = View.objects.filter_current_site() \
                            .filter_catalog(self.object.catalog) \
                            .filter_group(self.request.user) \
                            .filter_availability(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'views': views
        })
        return form_kwargs


class ProjectUpdateParentView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    form_class = ProjectUpdateParentForm
    permission_required = 'projects.change_project_object'

    def get_form_kwargs(self):
        projects = Project.objects.filter_user(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'projects': projects
        })
        return form_kwargs


class ProjectUpdateImportView(ProjectImportMixin, ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.import_project_object'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return HttpResponseRedirect(self.get_success_url())

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        method = request.POST.get('method')

        if method in ['upload_file', 'import_file', 'import_project']:
            response = getattr(self, method)()
        else:
            response = None

        if response is None:
            return render(request, 'core/error.html', {
                'title': _('Import error'),
                'errors': [_('There has been an error with your import.')]
            }, status=400)
        else:
            return response



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-03 15:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0026_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='question',
            name='optionsets',
        ),
        migrations.RemoveField(
            model_name='question',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='question',
            name='questionentity_ptr',
        ),
        migrations.RemoveField(
            model_name='questionentity',
            name='attribute_entity',
        ),
        migrations.RemoveField(
            model_name='questionentity',
            name='conditions',
        ),
        migrations.RemoveField(
            model_name='questionentity',
            name='subsection',
        ),
        migrations.DeleteModel(
            name='Question',
        ),
        migrations.DeleteModel(
            name='QuestionEntity',
        ),
    ]



# Generated by Django 2.2.13 on 2020-10-19 11:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0053_related_name'),
        ('tasks', '0031_related_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='catalogs',
            field=models.ManyToManyField(blank=True, help_text='The catalogs this task can be used with. An empty list implies that this task can be used with every catalog.', to='questions.Catalog', verbose_name='Catalogs'),
        ),
    ]
